@@ -110,6 +110,9 @@ void *create_directory_config(apr_pool_t *mp, char *path)
     /* Gsb Lookups */
     dcfg->gsb = NOT_SET_P;
 
+    /* Unicode Map */
+    dcfg->u_map = NOT_SET_P;
+
     /* Cache */
     dcfg->cache_trans = NOT_SET;
     dcfg->cache_trans_incremental = NOT_SET;
@@ -172,13 +175,13 @@ static void copy_rules_phase(apr_pool_t *mp,
                             char *my_error_msg = NULL;
                             const apr_array_header_t *tarr = NULL;
                             const apr_table_entry_t *telts = NULL;
-                            int i;
+                            int c;
 
                             tarr = apr_table_elts(rule->actionset->actions);
                             telts = (const apr_table_entry_t*)tarr->elts;
 
-                            for (i = 0; i < tarr->nelts; i++) {
-                                msre_action *action = (msre_action *)telts[i].val;
+                            for (c = 0; c < tarr->nelts; c++) {
+                                msre_action *action = (msre_action *)telts[c].val;
                                 if(strcmp("tag", action->metadata->name) == 0)  {
 
                                     int rc = msc_regexec(exceptions[j]->param_data,
@@ -483,6 +486,10 @@ void *merge_directory_configs(apr_pool_t *mp, void *_parent, void *_child)
     merged->gsb = (child->gsb == NOT_SET_P
         ? parent->gsb : child->gsb);
 
+    /* Unicode Map */
+    merged->u_map = (child->u_map == NOT_SET_P
+        ? parent->u_map : child->u_map);
+
     /* Cache */
     merged->cache_trans = (child->cache_trans == NOT_SET
         ? parent->cache_trans : child->cache_trans);
@@ -587,6 +594,9 @@ void init_directory_config(directory_config *dcfg)
     /* Gsb Lookup */
     if (dcfg->gsb == NOT_SET_P) dcfg->gsb = NULL;
 
+    /* Unicode Map */
+    if (dcfg->u_map == NOT_SET_P) dcfg->u_map = NULL;
+
     /* Cache */
     if (dcfg->cache_trans == NOT_SET) dcfg->cache_trans = MODSEC_CACHE_DISABLED;
     if (dcfg->cache_trans_incremental == NOT_SET) dcfg->cache_trans_incremental = 0;
@@ -1989,9 +1999,27 @@ static const char *cmd_geo_lookup_db(cmd_parms *cmd, void *_dcfg,
     return NULL;
 }
 
-/* Google safe browsing */
+/* Unicode CodePage */
 
-static const char *cmd_gsb_mal_lookup_db(cmd_parms *cmd, void *_dcfg,
+static const char *cmd_unicode_codepage(cmd_parms *cmd,
+                                        void *_dcfg, const char *p1)
+{
+    long val;
+
+    val = atol(p1);
+    if (val <= 0) {
+        return apr_psprintf(cmd->pool, "ModSecurity: Invalid setting for "
+                                       "SecUnicodeCodePage: %s", p1);
+    }
+
+    unicode_codepage = (unsigned long int)val;
+
+    return NULL;
+}
+
+/* Unicode Map */
+
+static const char *cmd_unicode_map(cmd_parms *cmd, void *_dcfg,
                                      const char *p1)
 {
     const char *filename = resolve_relative_path(cmd->pool, cmd->directive->filename, p1);
@@ -1999,7 +2027,24 @@ static const char *cmd_gsb_mal_lookup_db(cmd_parms *cmd, void *_dcfg,
     directory_config *dcfg = (directory_config *)_dcfg;
     if (dcfg == NULL) return NULL;
 
-    if (gsb_mal_init(dcfg, filename, &error_msg) <= 0) {
+    if (unicode_map_init(dcfg, filename, &error_msg) <= 0) {
+        return error_msg;
+    }
+
+    return NULL;
+}
+
+/* Google safe browsing */
+
+static const char *cmd_gsb_lookup_db(cmd_parms *cmd, void *_dcfg,
+                                     const char *p1)
+{
+    const char *filename = resolve_relative_path(cmd->pool, cmd->directive->filename, p1);
+    char *error_msg;
+    directory_config *dcfg = (directory_config *)_dcfg;
+    if (dcfg == NULL) return NULL;
+
+    if (gsb_db_init(dcfg, filename, &error_msg) <= 0) {
         return error_msg;
     }
 
@@ -2305,12 +2350,28 @@ const command_rec module_directives[] = {
 
     AP_INIT_TAKE1 (
         "SecGsbLookupDB",
-        cmd_gsb_mal_lookup_db,
+        cmd_gsb_lookup_db,
         NULL,
         RSRC_CONF,
         "database google safe browsing"
     ),
 
+    AP_INIT_TAKE1 (
+        "SecUnicodeCodePage",
+        cmd_unicode_codepage,
+        NULL,
+        CMD_SCOPE_MAIN,
+        "Unicode CodePage"
+    ),
+
+    AP_INIT_TAKE1 (
+        "SecUnicodeMapFile",
+        cmd_unicode_map,
+        NULL,
+        CMD_SCOPE_MAIN,
+        "Unicode Map file"
+    ),
+
     AP_INIT_TAKE1 (
         "SecGeoLookupDB",
         cmd_geo_lookup_db,
