@@ -84,7 +84,7 @@ apr_status_t input_filter(ap_filter_t *f, apr_bucket_brigade *bb_out,
         return APR_EGENERAL;
     }
 
-    if (chunk && !msr->txcfg->stream_inbody_inspection) {
+    if (chunk && (!msr->txcfg->stream_inbody_inspection || (msr->txcfg->stream_inbody_inspection && msr->if_stream_changed == 0))) {
         /* Copy the data we received in the chunk */
         bucket = apr_bucket_heap_create(chunk->data, chunk->length, NULL,
             f->r->connection->bucket_alloc);
@@ -114,6 +114,8 @@ apr_status_t input_filter(ap_filter_t *f, apr_bucket_brigade *bb_out,
         }
     } else if (msr->stream_input_data != NULL) {
 
+        msr->if_stream_changed = 0;
+
         bucket = apr_bucket_heap_create(msr->stream_input_data, msr->stream_input_length, NULL,
                 f->r->connection->bucket_alloc);
 
@@ -245,13 +247,10 @@ apr_status_t read_request_body(modsec_rec *msr, char **error_msg) {
                     *error_msg = apr_psprintf(msr->mp, "Request body is larger than the "
                             "configured limit (%ld).", msr->txcfg->reqbody_limit);
 
-                    seen_eos = 1;
-                    buflen = (msr->txcfg->reqbody_limit - msr->reqbody_length);
-
                 } else if ((msr->txcfg->is_enabled == MODSEC_DETECTION_ONLY) && (msr->txcfg->if_limit_action == REQUEST_BODY_LIMIT_ACTION_PARTIAL)){
 
-                    seen_eos = 1;
-                    buflen = (msr->txcfg->reqbody_limit - msr->reqbody_length);
+                    *error_msg = apr_psprintf(msr->mp, "Request body is larger than the "
+                            "configured limit (%ld).", msr->txcfg->reqbody_limit);
 
                 } else  {
 
@@ -280,13 +279,9 @@ apr_status_t read_request_body(modsec_rec *msr, char **error_msg) {
                         } else if ((msr->txcfg->is_enabled == MODSEC_ENABLED) && (msr->txcfg->if_limit_action == REQUEST_BODY_LIMIT_ACTION_PARTIAL)) {
                             *error_msg = apr_psprintf(msr->mp, "Request body no files data length is larger than the "
                                     "configured limit (%ld).", msr->txcfg->reqbody_no_files_limit);
-                            seen_eos = 1;
-
                         } else if ((msr->txcfg->is_enabled == MODSEC_DETECTION_ONLY) && (msr->txcfg->if_limit_action == REQUEST_BODY_LIMIT_ACTION_PARTIAL)) {
                             *error_msg = apr_psprintf(msr->mp, "Request body no files data length is larger than the "
                                     "configured limit (%ld).", msr->txcfg->reqbody_no_files_limit);
-                            seen_eos = 1;
-
                         } else {
                             *error_msg = apr_psprintf(msr->mp, "Request body no files data length is larger than the "
                                     "configured limit (%ld).", msr->txcfg->reqbody_no_files_limit);
@@ -913,7 +908,10 @@ apr_status_t output_filter(ap_filter_t *f, apr_bucket_brigade *bb_in) {
      * (full-buffering only).
      */
     if ((msr->of_skipping == 0)&&(!msr->of_partial)) {
-        inject_content_to_of_brigade(msr,f);
+        if(msr->of_stream_changed == 1) {
+            inject_content_to_of_brigade(msr,f);
+            msr->of_stream_changed = 0;
+        }
         prepend_content_to_of_brigade(msr, f);
 
         /* Inject content into response (append & buffering). */
