@@ -38,6 +38,7 @@ typedef struct msc_parm msc_parm;
 #include "msc_xml.h"
 #include "msc_geo.h"
 #include "msc_gsb.h"
+#include "msc_unicode.h"
 #include "re.h"
 
 #include "ap_config.h"
@@ -136,6 +137,10 @@ extern DSOLOCAL unsigned long int conn_read_state_limit;
 
 extern DSOLOCAL unsigned long int conn_write_state_limit;
 
+extern DSOLOCAL unsigned long int unicode_codepage;
+
+extern DSOLOCAL int *unicode_map_table;
+
 #define RESBODY_STATUS_NOT_READ         0   /* we were not configured to read the body */
 #define RESBODY_STATUS_ERROR            1   /* error occured while we were reading the body */
 #define RESBODY_STATUS_PARTIAL          2   /* partial body content available in the brigade */
@@ -233,6 +238,8 @@ struct modsec_rec {
     char                *stream_input_data;
     apr_size_t           stream_output_length;
     char                *stream_output_data;
+    unsigned int        of_stream_changed;
+    unsigned int        if_stream_changed;
 
     apr_array_header_t  *error_messages;
     apr_array_header_t  *alerts;
@@ -493,6 +500,9 @@ struct directory_config {
     /* Gsb Lookup */
     gsb_db              *gsb;
 
+    /* Unicode map */
+    unicode_map         *u_map;
+
     /* Cache */
     int                  cache_trans;
     int                  cache_trans_incremental;
@@ -554,8 +564,8 @@ struct msc_string {
 
 struct msc_parm {
     char                    *value;
-    unsigned int            pad_1;
-    unsigned int            pad_2;
+    int                     pad_1;
+    int                     pad_2;
 };
 
 /* Engine functions */
