@@ -1,25 +1,21 @@
 /*
- * ModSecurity for Apache 2.x, http://www.modsecurity.org/
- * Copyright (c) 2004-2010 Trustwave Holdings, Inc. (http://www.trustwave.com/)
- *
- * This product is released under the terms of the General Public Licence,
- * version 2 (GPLv2). Please refer to the file LICENSE (included with this
- * distribution) which contains the complete text of the licence.
- *
- * There are special exceptions to the terms and conditions of the GPL
- * as it is applied to this software. View the full text of the exception in
- * file MODSECURITY_LICENSING_EXCEPTION in the directory of this software
- * distribution.
- *
- * If any of the files related to licensing are missing or if you have any
- * other questions related to licensing please contact Trustwave Holdings, Inc.
- * directly using the email address support@trustwave.com.
- *
- */
+* ModSecurity for Apache 2.x, http://www.modsecurity.org/
+* Copyright (c) 2004-2011 Trustwave Holdings, Inc. (http://www.trustwave.com/)
+*
+* You may not use this file except in compliance with
+* the License.  You may obtain a copy of the License at
+*
+*     http://www.apache.org/licenses/LICENSE-2.0
+*
+* If any of the files related to licensing are missing or if you have any
+* other questions related to licensing please contact Trustwave Holdings, Inc.
+* directly using the email address security@modsecurity.org.
+*/
+
+#include "modsecurity.h"
 #include <ctype.h>
 #include <sys/stat.h>
 
-#include "mod_security2_config.h"
 #include "msc_multipart.h"
 #include "msc_util.h"
 #include "msc_parsers.h"
@@ -1346,7 +1342,7 @@ int multipart_get_arguments(modsec_rec *msr, char *origin, apr_table_t *argument
 /**
  *
  */
-char *multipart_reconstruct_urlencoded_body_sanitise(modsec_rec *msr) {
+char *multipart_reconstruct_urlencoded_body_sanitize(modsec_rec *msr) {
     multipart_part **parts;
     char *body;
     unsigned int body_len;
@@ -1384,7 +1380,7 @@ char *multipart_reconstruct_urlencoded_body_sanitise(modsec_rec *msr) {
              * in the memory.
              */
             if (msr->phase >= PHASE_LOGGING) {
-                if (apr_table_get(msr->arguments_to_sanitise, parts[i]->name) != NULL) {
+                if (apr_table_get(msr->arguments_to_sanitize, parts[i]->name) != NULL) {
                     memset(parts[i]->value, '*', strlen(parts[i]->value));
                 }
             }
