@@ -1,28 +1,24 @@
 /*
- * ModSecurity for Apache 2.x, http://www.modsecurity.org/
- * Copyright (c) 2004-2010 Trustwave Holdings, Inc. (http://www.trustwave.com/)
- *
- * This product is released under the terms of the General Public Licence,
- * version 2 (GPLv2). Please refer to the file LICENSE (included with this
- * distribution) which contains the complete text of the licence.
- *
- * There are special exceptions to the terms and conditions of the GPL
- * as it is applied to this software. View the full text of the exception in
- * file MODSECURITY_LICENSING_EXCEPTION in the directory of this software
- * distribution.
- *
- * If any of the files related to licensing are missing or if you have any
- * other questions related to licensing please contact Trustwave Holdings, Inc.
- * directly using the email address support@trustwave.com.
- *
- */
+* ModSecurity for Apache 2.x, http://www.modsecurity.org/
+* Copyright (c) 2004-2011 Trustwave Holdings, Inc. (http://www.trustwave.com/)
+*
+* You may not use this file except in compliance with
+* the License.  You may obtain a copy of the License at
+*
+*     http://www.apache.org/licenses/LICENSE-2.0
+*
+* If any of the files related to licensing are missing or if you have any
+* other questions related to licensing please contact Trustwave Holdings, Inc.
+* directly using the email address security@modsecurity.org.
+*/
+
 #include "msc_pcre.h"
 #include "apr_strings.h"
 
 /**
  * Releases the resources used by a single regular expression pattern.
  */
-apr_status_t msc_pcre_cleanup(msc_regex_t *regex) {
+static apr_status_t msc_pcre_cleanup(msc_regex_t *regex) {
     if (regex != NULL) {
         if (regex->pe != NULL) {
             free(regex->pe);
@@ -65,8 +61,9 @@ void *msc_pregcomp_ex(apr_pool_t *pool, const char *pattern, int options,
     }
     if (regex->re == NULL) return NULL;
 
-#ifdef WITH_PCRE_STUDY
+    #ifdef WITH_PCRE_STUDY
     pe = pcre_study(regex->re, 0, &errptr);
+    #endif
 
     /* Setup the pcre_extra record if pcre_study did not already do it */
     if (pe == NULL) {
@@ -76,7 +73,6 @@ void *msc_pregcomp_ex(apr_pool_t *pool, const char *pattern, int options,
         }
         memset(pe, 0, sizeof(pcre_extra));
     }
-#endif
 
 #ifdef PCRE_EXTRA_MATCH_LIMIT
     /* If match limit is available, then use it */
@@ -94,9 +90,7 @@ void *msc_pregcomp_ex(apr_pool_t *pool, const char *pattern, int options,
     }
 #endif /* MODSEC_PCRE_MATCH_LIMIT */
 #else
-#ifdef MODSEC_PCRE_MATCH_LIMIT
 #pragma message ( "This PCRE version does not support match limits!  Upgrade to at least PCRE v6.5." )
-#endif /* MODSEC_PCRE_MATCH_LIMIT */
 #endif /* PCRE_EXTRA_MATCH_LIMIT */
 
 #ifdef PCRE_EXTRA_MATCH_LIMIT_RECURSION
@@ -115,9 +109,7 @@ void *msc_pregcomp_ex(apr_pool_t *pool, const char *pattern, int options,
     }
 #endif /* MODSEC_PCRE_MATCH_LIMIT_RECURSION */
 #else
-#ifdef MODSEC_PCRE_MATCH_LIMIT_RECURSION
 #pragma message ( "This PCRE version does not support match recursion limits!  Upgrade to at least PCRE v6.5." )
-#endif /* MODSEC_PCRE_MATCH_LIMIT_RECURSION */
 #endif /* PCRE_EXTRA_MATCH_LIMIT_RECURSION */
 
     regex->pe = pe;
